
/* KRONOINO by Brainy_Baboon
 *  The original code was developed by Nathan Seidle for Sparkfun Electronics. It is open source and is available on GitHub https://github.com/sparkfun/BigTime
 *  I have subsequently modified the code to suit my purposes
 *  Some information on how to build the Kronoino can be found on my post on Hackaday.io https://hackaday.io/project/174942-kronoino-an-avr-powered-digital-wrist-watch
 *  This code is in the public domain, being originally developed by Nathan Seidle and licensed under the Beerware License
 *  Nathan wants a beer but I will settle for cup of tea, a pack of smokes and a nice conversation with anyone who feels benefited by this.
 *  I have added Nathan's Foreword to the BigTime code in the begining as it is. Thoughout the rest of the code I have modified Nathan's comments and added some of my own wherever I felt necessary  
 */


/*
 7-17-2011
 Spark Fun Electronics 2011
 Nathan Seidle
 
 This code is public domain but you buy me a beer if you use this and we meet someday (Beerware license).
 
 This is the firmware for BigTime, the wrist watch kit. It is based on an ATmega328 running with internal
 8MHz clock and external 32kHz crystal for keeping the time (aka RTC). The code and system have been tweaked
 to lower the power consumption of the ATmeg328 as much as possible. The watch currently uses about 
 1.2uA in idle (non-display) mode and about 13mA when displaying the time. With a 200mAh regular 
 CR2032 battery you should get 2-3 years of use!
 
 To compile and load this code onto your watch, select "Arduino Pro or Pro Mini 3.3V/8MHz w/ ATmega328" from
 the Boards menu. 
 
 If you're looking to save power in your own project, be sure to read section 9.10 of the ATmega328 
 datasheet to turn off all the bits of hardware you don't need.
 
 BigTime requires the Pro 8MHz bootloader with a few modifications:
 Internal 8MHz
 Clock div 8 cleared
 Brown out detect disabled
 BOOTRST set
 BOOSZ = 1024 
 This is to save power and open up the XTAL pins for use with a 38.786kHz external osc.
 
 So the fuse bits I get using AVR studio:
 HIGH 0xDA
 LOW 0xE2
 Extended 0xFF  
 
 3,600 seconds in an hour
 1 time check per hour, 2 seconds at 13mA
 3,598 seconds @ 1.2uA
 (3598 / 3600) * 0.0012mA + (2 / 3600) * 13mA = 0.0084mA used per hour
 
 200mAh / 0.0084mA = 23,809hr = 992 days = 2.7 years
 
 We can't use the standard Arduino delay() or delaymicrosecond() because we shut down timer0 to save power
 
 We turn off Brown out detect because it alone uses ~16uA.
 
 7-2-2011: Currently at 1.13uA
 
 7-4-2011: Let's wake up every 8 seconds instead of every 1 to save even more power
 Since we don't display seconds, this should be fine
 We are now at ~1.05uA on average
 
 7-17-2011: 1.09uA, portable with coincell power
 Jumps to 1.47uA every 8 seconds with system clock lowered to 1MHz
 Jumps to 1.37uA every 8 seconds with system clock at 8MHz
 Let's keep the internal clock at 8MHz since it doesn't seem to help to lower the internal clock.
 
 8-11-2011: Adding display color so that production can more easily know what code is on the 
 pre-programmed ATmega. 
 
 8-19-2011: Now we can print things like "red, gren, blue, yelo".
 
 7-12-2012: Added TV-B-Gone off codes. TV-B-Gone by Mitch Altman and Limor Fried. This project allows BigTime to
 turn off TVs by soldering an IR LED to Arduino pin 3 and GND. I updated the enclosure design files as well to
 allow the 5MM LED to fit inside the circumference of the enclosure.
 
 */

 


#include <avr/sleep.h> //Needed for sleep_mode
#include <avr/power.h> //Needed for powering down perihperals such as the ADC/TWI and Timers


//Set the 12hourMode to false for 24 hour mode. Set it to true for 12 hour mode.
boolean TwelveHourMode = true;


boolean show_only_hour= false;   // displays only hours or only minutes in setTime()

//Set this variable to change how long the time is shown on the watch face. In milliseconds so 1677 = 1.677 seconds
int show_time_length = 3000;
boolean show_the_time = false;

//You can set always_on to true and the display will stay on all the time
//This will drain the battery in about 15 hours 
boolean always_on = false;

long seconds = 55;
int minutes = 10;
int hours = 10;


int display_brightness = 15000; //A larger number makes the display more dim. This is the time delay between re-paints of the display

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//Pin definitions
int digit1 = 11; //AVR pin PB3
int digit2 = 12; //AVR pin PB4

int segA = 13; //AVR pin PB5
int segB = 14; //AVR pin PC0     We can use Analog pins as digital output pins but not as digital input because we shut down the ADC. You cannot attach a button to this pin 
int segC = 6; //AVR pin PD6      I changed the pin defs a bit to get a more convenient perfboard circuit. You can do the same to suit your circuit
int segD = 7; //AVR pin PD7
int segE = 8; //AVR pin PB0                                            see circuit diagram for help
int segF = 10; //AVR pin PB2
int segG = 9; //AVR pin PB1

int ampm = 15; //AVR pin PC1

int theButton = 2;  //AVR pin PD2
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//The very important 32.786kHz interrupt handler
SIGNAL(TIMER2_OVF_vect)        //Timer2 overflows every 8sec. This is set below in void setup()
{               
  seconds += 8; //We sleep for 8 seconds instead of 1 to save more power  
  //seconds++; //Use this if we are waking up every second  but you will also have to change the interrup overflow lines in setup

  //Update the minutes and hours variables
  minutes += seconds / 60; //Example: seconds = 2317, minutes = 58 + 38 = 96
  seconds %= 60; //seconds = 37
  hours += minutes / 60; //12 + (96 / 60) = 13
  minutes %= 60; //minutes = 36

  //Do we display 12 hour or 24 hour time?
  if(TwelveHourMode == true) 
  {
    //In 12 hour mode, hours go from 12 to 1 to 12.
    while(hours > 12)
    hours -= 12;
    
  }
  else
  {
    //In 24 hour mode, hours go from 0 to 23 to 0.
    while(hours > 23) hours -= 24;
  }
}

//The interrupt occurs when you push the button
SIGNAL(INT0_vect)
{
  //When you hit the button, we will need to display the time
  //if(show_the_time == false) 
  show_the_time = true;
}

void setup() 
{                
  //To reduce power, setup all pins as inputs with no pullups
  for(int x = 1 ; x < 18 ; x++){
    pinMode(x, INPUT);
    digitalWrite(x, LOW);
  }

  pinMode(theButton, INPUT); //This is the main button, tied to INT0
  digitalWrite(theButton, HIGH); //Enable internal pull up on button

  //These pins are used to control the display
  pinMode(segA, OUTPUT);
  pinMode(segB, OUTPUT);
  pinMode(segC, OUTPUT);
  pinMode(segD, OUTPUT);
  pinMode(segD, OUTPUT);
  pinMode(segE, OUTPUT);
  pinMode(segF, OUTPUT);
  pinMode(segG, OUTPUT);
 
  pinMode(digit1, OUTPUT);
  pinMode(digit2, OUTPUT);

  //Power down various bits of hardware to lower power usage  
  set_sleep_mode(SLEEP_MODE_PWR_SAVE);
  sleep_enable();

  //Shut off ADC, TWI, SPI, Timer0, Timer1

  ADCSRA &= ~(1<<ADEN); //Disable ADC
  ACSR = (1<<ACD); //Disable the analog comparator
  DIDR0 = 0x3F; //Disable digital input buffers on all ADC0-ADC5 pins
  DIDR1 = (1<<AIN1D)|(1<<AIN0D); //Disable digital input buffer on AIN1/0

  power_twi_disable();
  power_spi_disable();
   power_usart0_disable(); //Needed for serial.print disable this only after troubleshooting
  //power_timer0_disable(); //Needed for delay and millis() although disabling it would reduce power consumption we require the delay utility
  power_timer1_disable();   // disable Timer2
  //power_timer2_disable(); //Needed for asynchronous 32kHz operation. We will be using this 

  //Setup TIMER2
  TCCR2A = 0x00;
  TCCR2B = 0x00; //added on 17th sept. unsure
  delay(10); //added on 17th sept. unsure
  //TCCR2B = (1<<CS22)|(1<<CS20); //Set CLK/128 or overflow interrupt every 1s
  TCCR2B = (1<<CS22)|(1<<CS21)|(1<<CS20); //Set CLK/1024 or overflow interrupt every 8s
  ASSR = (1<<AS2); //Enable asynchronous operation
  delay(100); //added on 17th sept. unsure may help to stabilize crystal might have solved an issue. 
  TIMSK2 = (1<<TOIE2); //Enable the timer 2 interrupt
  delay(100); //added on 17th sept. unsure may help to stabilize crystal might have solved an issue.  
  //Setup external INT0 interrupt
  EICRA = (1<<ISC01); //Interrupt on falling edge
  EIMSK = (1<<INT0); //Enable INT0 interrupt. This is for generating an interrupt when we press the button 

  //System clock futzing
  //CLKPR = (1<<CLKPCE); //Enable clock writing
  //CLKPR = (1<<CLKPS3); //Divide the system clock by 256

 // Serial.begin(9600);  // added for troubleshooting
 //Serial.println("Setup ran sucessfully");    
 
  

    showTime(); //Show the current time for a few seconds

  sei(); //Enable global interrupts
}

void loop()                                     // deceptively simple loop function. 
{
  
  if(always_on == false)
    sleep_mode(); //Stop everything and go to sleep. Wake up if the Timer2 buffer overflows or if you hit the button

  if(show_the_time == true || always_on == true) 
  {
    
     
    while(digitalRead(theButton) == LOW) ; // as the button is tied to ground we read for logic low on the pin       
    delay(100);                            //Debounce
    while(digitalRead(theButton) == LOW) ; //Debounce

    /*Serial.print(hours, DEC);
     Serial.print(":");
     Serial.print(minutes, DEC);
     Serial.print(":");
     Serial.println(seconds, DEC);*/

    showTime(); //Show the current time for a few seconds on reset or power up

    //If you are STILL holding the button, then you must want to adjust the time
    if(digitalRead(theButton) == LOW) 
    setTime();

    show_the_time = false; //Reset the button variable
  }
}

void showTime() 
{


  long startTime = millis();
  show_only_hour = true;
  while( (millis() - startTime) < show_time_length ) 
  {
     if(show_only_hour==true)
      {
        show_only_hour=false; 
        while( (millis() - startTime) < (show_time_length/2) ) 
        {
         displayNumber(hours);     // display the hours for half of time
         delayMicroseconds(display_brightness); //Time delay between repaints of display; percieved as brightness
        
        }
    
      }

      else
      {
        while( (millis() - startTime) > (show_time_length/2) && (millis() - startTime) < show_time_length )   //we display only the minutes first because show_only_hour is set to false                          
       {                
        displayNumber(minutes);            
        delayMicroseconds(display_brightness);      //Time delay between repaints of display; percieved as brightness
       }
      }
      
    
  }
          
  

}



void setTime(void)
{

  
  int idleMiliseconds = 0;   // timeout counter when it reaches 2 sec we exit setTime and re-enter normal operations
 
  int buttonHold = 0; 
  //This counts the number of times you are holding the button down consecutively
  //Once we notice you're really holding the button down a lot, we will speed up the minute counter

  while(idleMiliseconds < 2000) 
  {

    cli(); //Disable interrupts
           // Interrupts update time during normal operations they need to be diabled while we are setting the time 

    //Update the minutes and hours variables
    minutes += seconds / 60; //Example: seconds = 2317, minutes = 58 + 38 = 96
    seconds %= 60; //seconds = 37
    hours += minutes / 60; //12 + (96 / 60) = 13
    minutes %= 60; //minutes = 36

    //Support for both 12 hour mode and 24 hour mode
    if(TwelveHourMode == true) 
    {
      //In 12 hour mode, hours go from 12 to 1 to 12.
      while(hours > 12)
       hours -= 12;  
    
    }
    else 
    {  
      //In 24 hour mode, hours go from 0 to 23 to 0.
      while(hours > 23) hours -= 24;
    }

    sei(); //Resume interrupts

   

      if(show_only_hour==true)                             // we want to display on the hours while changing the hours
      {
        
        for(int x = 0 ; x < 10 ; x++) 
        {
          
          displayNumber(hours); 
          delayMicroseconds(display_brightness); //Time delay between repaints of display; percieved as brightness
        }
      }

      else
      {
    for(int x = 0 ; x < 10 ; x++)            //we display only the minutes first because show_only_hour is set to false as default and we are going to change the minutes first 
    {  
      displayNumber(minutes); 
      delayMicroseconds(display_brightness); //Time delay between repaints of display; percieved as brightness
    }
      }
    //Checks if you still pressing the button, if yes then we change the time
    if(digitalRead(theButton) == LOW) 
    {
      
      idleMiliseconds = 0;

      buttonHold++;
      if(buttonHold < 10) //10 = 2 seconds
        minutes++;        //Advance the minutes
      else 
      {
       
        show_only_hour= true;  //show only the hours while advancing the hours
        minutes+= 10; //we increase 10 minutes at each iteration while displaying only hours.
      }
    }
    else
      buttonHold = 0;

    idleMiliseconds += 200;
  } // end of while loop

  show_only_hour= false;    // reset the counter
}

//This is a not-so-accurate delay routine
//Calling fake_msdelay(100) will delay for about 100ms
//Assumes 8MHz clock
/*void fake_msdelay(int x){                                                           // we can use this delay routine because, lets face it, we aren't displaying seconds and we are not going to be changing the time so often that seconds accuracy will matter
 for( ; x > 0 ; x--)
 fake_usdelay(1000);
 }*/

//This is a not-so-accurate delay routine
//Calling fake_usdelay(100) will delay for about 100us
//Assumes 8MHz clock
/*void fake_usdelay(int x){
 for( ; x > 0 ; x--) {
 __asm__("nop\n\t"); 
 __asm__("nop\n\t"); 
 __asm__("nop\n\t"); 
 __asm__("nop\n\t"); 
 __asm__("nop\n\t"); 
 __asm__("nop\n\t"); 
 __asm__("nop\n\t"); 
 }
 }*/


void displayNumber(int toDisplay) 
{
  
#define DIGIT_ON  HIGH                                     //These are the common pins on the seven segment displays. As my seven segment displays are common anode I need to apply positive voltage or logic HIGH to switch it on
#define DIGIT_OFF  LOW                                     // For a common cathode display the reverse is true. Applying logic LOW to the common pin will switch it on  

  for(int digit = 2 ; digit > 0 ; digit--)
  {
     // Since we are multiplexing we can only display one digit at a time. We use persistence of vision to create the impression of two(or more digits)
    //Turn one seven segment by powering up it common anode for a short amount of time(appox. 5000 microseconds), then we move on to the next seven segment.
    // All this happens too quickly for the human eye to distinguish 
    switch(digit)
    {
    case 1:
      digitalWrite(digit1, DIGIT_ON);
      break;
    case 2:
      digitalWrite(digit2, DIGIT_ON);
      break;
    
    }
    //Serial.println(toDisplay);

    //Now display this digit
    if(digit > 1)
      {lightNumber(toDisplay % 10);  // We display a single digit on the tens' seven segment 
      //Serial.println(toDisplay % 10);
      }
    else if(digit == 1)  //Special case on first digit, the watch will not display a zero before a single digit, for example 8 hours 30 min will be displayed as 8 hours and not as 08 hours
      {
        if( (toDisplay % 10) != 0) //verifying it is not zero
      { 
        lightNumber(toDisplay % 10); //We display a single digit on the ones' seven segment 
      // Serial.print("Single Digit:");
      // Serial.println(toDisplay % 10);  // Serial.print were added to check if code was working properly
    }
    }

    toDisplay /= 10;

    delayMicroseconds(5000); //This displays a digit for 5ms
    //If you set this too long, the display will start to flicker. Set it to 25000 and you can see multiplexing magic in slow motion

    //Turn off all segments
    lightNumber(10);

    //Turn off all digits
    digitalWrite(digit1, DIGIT_OFF);
    digitalWrite(digit2, DIGIT_OFF);
   
    
  }

}



//Given a number, turns on those segments
//If number == 10, then turn off all segments
void lightNumber(int numberToDisplay)                        // We turn on the right segments for each number. This function is basically like a decoder
                                                    
{

#define SEGMENT_ON  LOW                                      // For my common anode displays I need to apply Logic LOW on a segment pin to turn it on. Remember we applied logic high on the common pin. This completes the circuit 
#define SEGMENT_OFF HIGH                                     // For common cathode displays the logic will be reverse. Applying logic HIGH while simultaneously applying logic LOW on the common pin will turn on a segment

  /*
Segments
   -  A
   F / / B
   -  G
   E / / C
   -  D
   */

  switch (numberToDisplay)
  
 {

  case 0:
    digitalWrite(segA, SEGMENT_ON);
    digitalWrite(segB, SEGMENT_ON);
    digitalWrite(segC, SEGMENT_ON);
    digitalWrite(segD, SEGMENT_ON);
    digitalWrite(segE, SEGMENT_ON);
    digitalWrite(segF, SEGMENT_ON);
    break;

  case 1:
    digitalWrite(segB, SEGMENT_ON);
    digitalWrite(segC, SEGMENT_ON);
    break;

  case 2:
    digitalWrite(segA, SEGMENT_ON);
    digitalWrite(segB, SEGMENT_ON);
    digitalWrite(segD, SEGMENT_ON);
    digitalWrite(segE, SEGMENT_ON);
    digitalWrite(segG, SEGMENT_ON);
    break;

  case 3:
    digitalWrite(segA, SEGMENT_ON);
    digitalWrite(segB, SEGMENT_ON);
    digitalWrite(segC, SEGMENT_ON);
    digitalWrite(segD, SEGMENT_ON);
    digitalWrite(segG, SEGMENT_ON);
    break;

  case 4:
    digitalWrite(segB, SEGMENT_ON);
    digitalWrite(segC, SEGMENT_ON);
    digitalWrite(segF, SEGMENT_ON);
    digitalWrite(segG, SEGMENT_ON);
    break;

  case 5:
    digitalWrite(segA, SEGMENT_ON);
    digitalWrite(segC, SEGMENT_ON);
    digitalWrite(segD, SEGMENT_ON);
    digitalWrite(segF, SEGMENT_ON);
    digitalWrite(segG, SEGMENT_ON);
    break;

  case 6:
    digitalWrite(segA, SEGMENT_ON);
    digitalWrite(segC, SEGMENT_ON);
    digitalWrite(segD, SEGMENT_ON);
    digitalWrite(segE, SEGMENT_ON);
    digitalWrite(segF, SEGMENT_ON);
    digitalWrite(segG, SEGMENT_ON);
    break;

  case 7:
    digitalWrite(segA, SEGMENT_ON);
    digitalWrite(segB, SEGMENT_ON);
    digitalWrite(segC, SEGMENT_ON);
    break;

  case 8:
    digitalWrite(segA, SEGMENT_ON);
    digitalWrite(segB, SEGMENT_ON);
    digitalWrite(segC, SEGMENT_ON);
    digitalWrite(segD, SEGMENT_ON);
    digitalWrite(segE, SEGMENT_ON);
    digitalWrite(segF, SEGMENT_ON);
    digitalWrite(segG, SEGMENT_ON);
    break;

  case 9:
    digitalWrite(segA, SEGMENT_ON);
    digitalWrite(segB, SEGMENT_ON);
    digitalWrite(segC, SEGMENT_ON);
    digitalWrite(segD, SEGMENT_ON);
    digitalWrite(segF, SEGMENT_ON);
    digitalWrite(segG, SEGMENT_ON);
    break;

  case 10:
    digitalWrite(segA, SEGMENT_OFF);
    digitalWrite(segB, SEGMENT_OFF);
    digitalWrite(segC, SEGMENT_OFF);
    digitalWrite(segD, SEGMENT_OFF);
    digitalWrite(segE, SEGMENT_OFF);
    digitalWrite(segF, SEGMENT_OFF);
    digitalWrite(segG, SEGMENT_OFF);
    break;


  }
}



//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=




                 /* troubleshoot Advice
                  * 8 Sept 2020 Try checking if all the segments are turned off in the code before displaying a new number 
                  * 21 Sept changed hours increment code. Maybe be problematic. Consider revising
                  * 18 Sept added delays when setting up Timer2 to allow for the crystal to stabilze. May have solved the problem.
                  




                  */



